import { InMemoryCache, ApolloClient, useQuery, gql, ApolloProvider } from '@apollo/client';
export { gql, useMutation, useQuery } from '@apollo/client';
import React, { useState, useEffect, useContext, createContext } from 'react';
import styled from 'styled-components';
import { Icon, Button, Dropdown, DropdownImageInput, DropdownImageItem, Header, DropdownIconItem, Main, Navbar, Footer } from 'bonde-components';
import { CrossStorageClient } from 'cross-storage';
import axios from 'axios';
import logger$1 from 'pino';

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _taggedTemplateLiteralLoose(strings, raw) {
  if (!raw) {
    raw = strings.slice(0);
  }

  strings.raw = raw;
  return strings;
}

var SessionStorage = /*#__PURE__*/function () {
  function SessionStorage(uri) {
    this.session = {};
    this.authenticated = false; // Init session client on cross-storage

    this.storage = new CrossStorageClient(uri, {
      timeout: 10000
    });
  }

  var _proto = SessionStorage.prototype;

  _proto.login = function login(user) {
    var _this = this;

    return this.storage.onConnect().then(function () {
      _this.authenticated = true;
      _this.token = user.token;
      return _this.storage.set('auth', JSON.stringify(user));
    });
  };

  _proto.logout = function logout() {
    var _this2 = this;

    return this.storage.onConnect().then(function () {
      return _this2.storage.del('auth', 'community').then(function () {
        _this2.token = undefined;
        _this2.session = {};
        return Promise.resolve();
      });
    });
  };

  _proto.getAsyncSession = function getAsyncSession() {
    var _this3 = this;

    return this.storage.onConnect().then(function () {
      return _this3.storage.get('auth', 'community');
    }).then(function (args) {
      var authJson = args[0];
      var communityJson = args[1];

      if (authJson) {
        var dataSession = {
          token: JSON.parse(authJson).token,
          community: communityJson ? JSON.parse(communityJson) : {}
        };
        return Promise.resolve(dataSession);
      }
    });
  };

  _proto.setAsyncItem = function setAsyncItem(key, value) {
    var _this4 = this;

    return this.storage.onConnect().then(function () {
      return _this4.storage.set(key, JSON.stringify(value));
    });
  };

  return SessionStorage;
}();

var createGraphQLClient = (function (uri, session) {
  if (session === void 0) {
    session = {};
  }

  var apolloClientOpts = {
    uri: uri,
    cache: new InMemoryCache(),
    headers: {
      'client-name': 'bonde-core-tools [web]',
      'client-version': '1.0.0'
    }
  };

  if (session.token) {
    apolloClientOpts.headers['authorization'] = "Bearer " + session.token;
  }

  return new ApolloClient(apolloClientOpts);
});

function _templateObject() {
  var data = _taggedTemplateLiteralLoose(["\n  query CurrentUser {\n    currentUser {\n      id\n      firstName\n      lastName\n      email\n      createdAt\n      avatar\n      isAdmin\n    }\n  }\n"]);

  _templateObject = function _templateObject() {
    return data;
  };

  return data;
}
var FETCH_USER = /*#__PURE__*/gql( /*#__PURE__*/_templateObject());
var FetchUser = (function (_ref) {
  var children = _ref.children,
      Loading = _ref.loading,
      logout = _ref.logout;

  var _useQuery = useQuery(FETCH_USER),
      loading = _useQuery.loading,
      error = _useQuery.error,
      data = _useQuery.data;

  if (loading) return React.createElement(Loading, {
    fetching: "user"
  });

  if (error || !data.currentUser) {
    console.log('error', {
      error: error,
      data: data
    });
    logout();
  }

  return children({
    user: data.currentUser
  });
});

function _templateObject$1() {
  var data = _taggedTemplateLiteralLoose(["\n  query RelatedCommunities {\n    communities {\n      id\n      name\n      city\n      description\n      image\n      created_at\n      updated_at\n      mailchimp_api_key\n      mailchimp_list_id\n      mailchimp_group_id\n      fb_link\n      twitter_link\n      facebook_app_id\n      email_template_from\n      modules\n      recipient {\n        id\n        pagarme_recipient_id\n        transfer_day: recipient(path: \"transfer_day\")\n        transfer_interval: recipient(path: \"transfer_interval\")\n        transfer_enabled: recipient(path: \"transfer_enabled\")\n        bank_account: recipient(path: \"bank_account\")\n      }\n    }\n  }\n"]);

  _templateObject$1 = function _templateObject() {
    return data;
  };

  return data;
}
var FETCH_RELATED_COMMUNITIES = /*#__PURE__*/gql( /*#__PURE__*/_templateObject$1());
var FetchCommunities = (function (_ref) {
  var children = _ref.children,
      Loading = _ref.loading;

  var _useQuery = useQuery(FETCH_RELATED_COMMUNITIES),
      loading = _useQuery.loading,
      error = _useQuery.error,
      data = _useQuery.data;

  if (loading) return React.createElement(Loading, {
    fetching: "communities"
  });

  if (error || !data.communities) {
    console.log('error', {
      error: error,
      data: data
    });
    return children({
      communities: []
    });
  }

  return children({
    communities: data.communities
  });
});

var nextURI = (function (uri) {
  return uri + "?next=" + window.location.href;
});

var bondeConfig = {
  development: {
    // Modules URLs
    accounts: 'http://accounts.bonde.devel:5000/login',
    settings: 'http://admin-canary.bonde.devel:5001/community/settings',
    mobilization: 'http://app.bonde.devel:5002/mobilizations',
    chatbot: 'http://chatbot.bonde.devel:5003',
    redes: 'http://redes.bonde.devel:5004',
    // Essentials URLs
    crossStorage: 'http://cross-storage.bonde.devel',
    apiGraphql: 'https://api-graphql.staging.bonde.org/v1/graphql'
  },
  staging: {
    // Modules URLs
    accounts: 'https://accounts.staging.bonde.org/login',
    mobilization: 'https://app.staging.bonde.org/mobilizations',
    settings: 'https://admin-canary.staging.bonde.org/community/settings',
    chatbot: 'https://chatbot.staging.bonde.org',
    redes: 'https://redes.staging.bonde.org',
    // Essentials URLs
    crossStorage: 'https://cross-storage.staging.bonde.org',
    apiGraphql: 'https://api-graphql.staging.bonde.org/v1/graphql'
  },
  production: {
    // Modules URLs
    accounts: 'https://accounts.bonde.org/login',
    mobilization: 'https://app.bonde.org/mobilizations',
    settings: 'https://admin-canary.bonde.org/community/settings',
    chatbot: 'https://chatbot.bonde.org',
    redes: 'https://redes.bonde.org',
    // Essentials URLs
    crossStorage: 'https://cross-storage.bonde.org',
    apiGraphql: 'https://api-graphql.bonde.org/v1/graphql'
  }
};

var settings = (function (environment) {
  var defaultConfig = bondeConfig.development;
  var environmentConfig = bondeConfig[environment];
  return Object.assign({}, defaultConfig, environmentConfig);
});

/**
 * Responsible to control session used on cross-storage
 **/

var Context = /*#__PURE__*/createContext({
  signing: true,
  isLogged: false
});

var SessionProvider = function SessionProvider(_ref) {
  var children = _ref.children,
      environment = _ref.environment,
      extraConfig = _ref.extraConfig,
      fetchData = _ref.fetchData,
      Loading = _ref.loading;

  // 0. Start controller states
  var _useState = useState(false),
      redirecting = _useState[0],
      setRedirecting = _useState[1];

  var _useState2 = useState(undefined),
      token = _useState2[0],
      setToken = _useState2[1];

  var _useState3 = useState(true),
      signing = _useState3[0],
      setSigning = _useState3[1];

  var _useState4 = useState(0),
      refetch = _useState4[0],
      setRefetch = _useState4[1];

  var _useState5 = useState(undefined),
      community = _useState5[0],
      setCommunity = _useState5[1]; // 1. Merge settings to start BondeSessionProvider requests


  var config = Object.assign({}, settings(environment), extraConfig); // 2. Create a SessionStorage client connect on cross-storage

  var storage = new SessionStorage(config.crossStorage);

  var fetchSession = function fetchSession() {
    // Function to fetch a shared session
    storage.getAsyncSession().then(function (_temp) {
      var _ref2 = _temp === void 0 ? {} : _temp,
          stoken = _ref2.token,
          scommunity = _ref2.community;

      if (!stoken) throw Error('unauthorized');
      setToken(stoken);
      setSigning(false);
      setCommunity(!!Object.keys(scommunity || {}).length ? scommunity : undefined);
      return Promise.resolve();
    })["catch"](function (err) {
      // TODO: change url admin-canary
      if (err && err.message === 'unauthorized') {
        if (config && config.accounts) {
          window.location.href = nextURI(config.accounts);
        }

        setToken(undefined);
        setSigning(false);
      } else {
        // reload fetchSession when error not authorized
        console.log('err', err.message);
        setRefetch(refetch + 1);
        if (refetch < 3) fetchSession();
      }
    });
  };

  useEffect(function () {
    if (!token) return fetchSession(); // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [token]);

  var login = function login(user) {
    return storage.login(user);
  };

  var logout = function logout() {
    return storage.logout().then(function () {
      if (config && config.accounts) {
        window.location.href = config.accounts;
      }

      return Promise.resolve();
    })["catch"](function (err) {
      return console.log('err', err);
    });
  }; // TODO: Tratar erros */


  var onChange = function onChange(_ref3) {
    var community = _ref3.community,
        url = _ref3.url;

    if (!!community) {
      storage.setAsyncItem('community', community);
      setCommunity(community);
    }

    if (!!url) {
      window.location.href = url;
      setRedirecting(true);
    }
  };

  var onChangeAsync = function onChangeAsync(props) {
    try {
      return Promise.resolve(new Promise(function (resolve) {
        onChange(props);
        return resolve();
      }));
    } catch (e) {
      return Promise.reject(e);
    }
  };

  var session = {
    storage: storage,
    signing: signing,
    token: token,
    community: community,
    onChange: onChange,
    onChangeAsync: onChangeAsync,
    login: login,
    logout: logout,
    config: config,
    isLogged: !!token,
    loading: Loading
  };
  return signing ? React.createElement(Loading, {
    fetching: "session"
  }) : redirecting ? React.createElement(Loading, {
    fetching: "module"
  }) : React.createElement(ApolloProvider, {
    client: createGraphQLClient(config.apiGraphql, session)
  }, fetchData && session.isLogged ? React.createElement(FetchUser, {
    loading: Loading,
    logout: logout
  }, function (user) {
    return React.createElement(FetchCommunities, {
      loading: Loading
    }, function (communities) {
      return React.createElement(Context.Provider, {
        value: _extends({}, session, user, communities)
      }, children);
    });
  }) : fetchData && !session.isLogged ? React.createElement(Loading, {
    fetching: "redirect"
  }) : React.createElement(Context.Provider, {
    value: session
  }, children));
};

SessionProvider.defaultProps = {
  environment: 'development',
  fetchData: false
};
var useSession = function useSession() {
  return useContext(Context);
};

function _templateObject$2() {
  var data = _taggedTemplateLiteralLoose(["\n  display: flex;\n  flex-direction: row;\n  justify-content: flex-end;\n\n  button {\n    border: none;\n    padding: 0;\n\n    ", "\n  }\n\n  ", " {\n    margin-left: 10px;\n  }\n"]);

  _templateObject$2 = function _templateObject() {
    return data;
  };

  return data;
}
var MenuStyled = /*#__PURE__*/styled.div( /*#__PURE__*/_templateObject$2(), function (props) {
  return props.inverted && "\n      &.active {\n        " + Icon + " {\n          &.stroke {\n            path {\n              stroke: #ee0099;\n            }\n          }\n          &.fill {\n            path {\n              fill: #ee0099;\n            }\n          }\n        }\n      }\n    ";
}, Icon);
MenuStyled.defaultProps = {
  inverted: false
};

var MenuItem = function MenuItem(_ref) {
  var icon = _ref.icon,
      title = _ref.title,
      onClick = _ref.onClick,
      inverted = _ref.inverted,
      active = _ref.active;
  return React.createElement(Button, {
    className: active ? 'active' : null,
    dark: !inverted,
    light: inverted,
    onClick: onClick,
    title: title
  }, React.createElement(Icon, {
    name: icon,
    size: "small"
  }));
};

MenuItem.defaultProps = {
  inverted: false,
  active: false
};
var items = {
  settings: ['Settings', 'Configurações'],
  mobilization: ['Window', 'Mobilizações'],
  redes: ['Network', 'Redes'],
  chatbot: ['Bot', 'Chatbot']
};

var CommunityMenu = function CommunityMenu(_ref2) {
  var community = _ref2.community,
      inverted = _ref2.inverted;

  var _useSession = useSession(),
      onChange = _useSession.onChange,
      config = _useSession.config;

  var modules = community.modules;

  var click = function click(url) {
    return function () {
      try {
        return Promise.resolve(onChange({
          community: community,
          url: url
        })).then(function () {});
      } catch (e) {
        return Promise.reject(e);
      }
    };
  };

  return React.createElement(MenuStyled, {
    inverted: inverted
  }, Object.keys(modules).filter(function (key) {
    return !!modules[key];
  }).map(function (key) {
    var moduleHost = new URL('', config[key]).host;
    var baseHost = new URL('', window.location.href).host;
    return React.createElement(MenuItem, {
      active: moduleHost === baseHost,
      inverted: inverted,
      key: key,
      icon: items[key][0],
      title: items[key][1],
      onClick: click(config[key])
    });
  }));
};

CommunityMenu.defaultProps = {
  inverted: false
};

function _templateObject$3() {
  var data = _taggedTemplateLiteralLoose(["\n  display: flex;\n  flex-direction: row;\n\n  ", " {\n    img {\n      border-radius: 30px;\n    }\n\n    &:hover {\n      background-color: rgb(247, 247, 247);\n    }\n  }\n"]);

  _templateObject$3 = function _templateObject() {
    return data;
  };

  return data;
}
var WithMenu = /*#__PURE__*/styled.div( /*#__PURE__*/_templateObject$3(), DropdownImageItem);

var toItem = function toItem(c) {
  return !!c ? {
    img: {
      src: c.image || "https://via.placeholder.com/100?text=" + c.name.substring(0, 1),
      alt: c.name
    },
    label: c.name,
    id: c.id
  } : undefined;
};

var CommunitiesDropdown = function CommunitiesDropdown() {
  var _useSession = useSession(),
      community = _useSession.community,
      communities = _useSession.communities,
      onChange = _useSession.onChange;

  return React.createElement(WithMenu, null, React.createElement(Dropdown, {
    placeholder: "Selecione uma comunidade",
    item: toItem(community),
    items: communities.map(toItem),
    onSelect: function onSelect(value) {
      onChange({
        community: communities.find(function (c) {
          return c.id === value.id;
        })
      });
    },
    dropdownInput: DropdownImageInput,
    dropdownItem: DropdownImageItem
  }), community && React.createElement(CommunityMenu, {
    community: community,
    inverted: true
  }));
};

var UserDropdown = function UserDropdown() {
  var _useSession = useSession(),
      user = _useSession.user,
      logout = _useSession.logout;

  var name = user.firstName + " " + user.lastName;
  return React.createElement(Dropdown, {
    selectable: false,
    direction: "right",
    placeholder: name,
    items: [{
      clickable: false,
      img: {
        src: user.avatar || 'http://via.placeholder.com/35x35?text=U',
        alt: name
      },
      label: name,
      email: user.email,
      render: function render(_ref) {
        var value = _ref.value;
        var img = value.img,
            label = value.label,
            email = value.email;
        return React.createElement(DropdownImageItem, {
          value: {
            img: img,
            label: React.createElement("div", null, React.createElement(Header.h4, null, label), React.createElement(Header.h5, null, email))
          }
        });
      }
    }, // {
    //   icon: 'User',
    //   label: 'Perfil',
    //   name: 'profile',
    // },
    {
      icon: 'Close',
      label: 'Logout',
      name: 'logout'
    }],
    onSelect: function onSelect(_ref2) {
      var name = _ref2.name;
      if (name === 'logout') return logout();
      if (name === 'profile') return 'asdasdasd';
    },
    dropdownItem: DropdownIconItem
  });
};

function _templateObject$4() {
  var data = _taggedTemplateLiteralLoose(["\n  display: flex;\n  flex-grow: 1;\n  background-color: ", ";\n"]);

  _templateObject$4 = function _templateObject() {
    return data;
  };

  return data;
}
var Content = /*#__PURE__*/styled.div( /*#__PURE__*/_templateObject$4(), function (props) {
  return props.bgColor;
});

var BondeSessionUI = function BondeSessionUI(_ref) {
  var children = _ref.children,
      bgColor = _ref.bgColor,
      disableNavigation = _ref.disableNavigation,
      indexRoute = _ref.indexRoute,
      LanguageTool = _ref.languageTool;
  return React.createElement(Main, null, React.createElement(Navbar, {
    indexRoute: indexRoute,
    brand: disableNavigation ? 'default' : 'small'
  }, disableNavigation ? React.createElement("div", null) : React.createElement(CommunitiesDropdown, null), React.createElement(UserDropdown, null)), React.createElement(Content, {
    bgColor: bgColor || 'rgb(247,247,247)'
  }, children), LanguageTool ? React.createElement(Footer, null, React.createElement(LanguageTool, null)) : React.createElement(Footer, null));
};

BondeSessionUI.defaultProps = {
  disableNavigation: false
};

// A type of promise-like that resolves synchronously and supports only one observer

const _iteratorSymbol = /*#__PURE__*/ typeof Symbol !== "undefined" ? (Symbol.iterator || (Symbol.iterator = Symbol("Symbol.iterator"))) : "@@iterator";

const _asyncIteratorSymbol = /*#__PURE__*/ typeof Symbol !== "undefined" ? (Symbol.asyncIterator || (Symbol.asyncIterator = Symbol("Symbol.asyncIterator"))) : "@@asyncIterator";

// Asynchronously call a function and send errors to recovery continuation
function _catch(body, recover) {
	try {
		var result = body();
	} catch(e) {
		return recover(e);
	}
	if (result && result.then) {
		return result.then(void 0, recover);
	}
	return result;
}

var logger = /*#__PURE__*/logger$1();

var log = /*#__PURE__*/logger.child({
  module: 'geolocation'
});

var getCityStateAndZipcode = function getCityStateAndZipcode(addressComponents) {
  var state = '';
  var city = '';
  var zipcode = '';
  var country = '';
  addressComponents.forEach(function (_ref) {
    var types = _ref.types,
        shortName = _ref.short_name;

    if (types.includes('postal_code')) {
      zipcode = shortName;
    }

    if (types.includes('administrative_area_level_1')) {
      state = shortName;
    }

    if (types.includes('administrative_area_level_2')) {
      city = shortName;
    } else if (types.includes('locality')) {
      city = shortName;
    }

    if (types.includes('country')) {
      country = shortName;
    }
  });

  if (country !== 'BR') {
    city = 'Internacional';
    zipcode = 'Internacional';
    state = 'INT';
  }

  return [state, city, zipcode];
};

var processGeolocation = function processGeolocation(userEmail, searchAddress, data) {
  if (data && data.status === 'OK') {
    var _data$results = data.results,
        _data$results$ = _data$results[0],
        _data$results$$geomet = _data$results$.geometry.location,
        _data$results$$geomet2 = _data$results$$geomet.lat,
        lat = _data$results$$geomet2 === void 0 ? '' : _data$results$$geomet2,
        _data$results$$geomet3 = _data$results$$geomet.lng,
        lng = _data$results$$geomet3 === void 0 ? '' : _data$results$$geomet3,
        addressComponents = _data$results$.address_components,
        _data$results$$format = _data$results$.formatted_address,
        address = _data$results$$format === void 0 ? '' : _data$results$$format;

    var _getCityStateAndZipco = getCityStateAndZipcode(addressComponents),
        state = _getCityStateAndZipco[0],
        city = _getCityStateAndZipco[1],
        zipcode = _getCityStateAndZipco[2];

    var latitude = typeof lat === 'string' ? lat : lat.toFixed(3);
    var longitude = typeof lng === 'string' ? lng : lng.toFixed(3);
    var _i = {
      latitude: latitude,
      longitude: longitude,
      address: address,
      state: state,
      city: city,
      cep: zipcode
    }; // log(i);

    log.info('returned valid individual geolocation data');
    return _i;
  }

  if (data && data.status === 'ZERO_RESULTS') {
    log.warn("google maps return with zero result (email, address): '" + userEmail + "', " + searchAddress);
  }

  var i = {
    latitude: null,
    longitude: null,
    address: "Endere\xE7o n\xE3o encontrado - " + searchAddress,
    state: null,
    city: 'ZERO_RESULTS',
    cep: 'ZERO_RESULTS'
  };
  return i;
};
var getGoogleGeolocation = function getGoogleGeolocation(address, email) {
  try {
    return Promise.resolve(_catch(function () {
      log.info("requesting google with address " + address + "...");
      return Promise.resolve(axios.post('https://maps.googleapis.com/maps/api/geocode/json', undefined, {
        params: {
          address: address,
          key: process.env.GOOGLE_MAPS_API_KEY
        }
      })).then(function (response) {
        log.info('google maps responded!');

        if (response.data && response.data.error_message) {
          log.warn("google maps response failed (email, address): '" + email + "', " + address + " %o", response.data.error_message);
          return processGeolocation(email, address, undefined);
        }

        return processGeolocation(email, address, response.data);
      });
    }, function (e) {
      log.warn("google maps response failed (email, address): '" + email + "', " + address + " %o", e);
      return processGeolocation(email, address, undefined);
    }));
  } catch (e) {
    return Promise.reject(e);
  }
};
var index = (function (_ref2) {
  var state = _ref2.state,
      city = _ref2.city,
      cep = _ref2.cep,
      address = _ref2.address,
      neighborhood = _ref2.neighborhood,
      email = _ref2.email;
  var a = address ? address + "," : '';
  var n = neighborhood ? neighborhood + "," : '';
  var c = city ? city + "," : '';
  var s = state ? state + "," : '';
  var z = cep ? cep + ',BR' : '';
  var composeSearchAddress = a + n + c + s + z;
  return Promise.resolve(getGoogleGeolocation(composeSearchAddress, email));
});

export { SessionProvider as BondeSessionProvider, BondeSessionUI, CommunityMenu, index as getGeolocation, useSession };
//# sourceMappingURL=bonde-core-tools.esm.js.map
